@@ -4,63 +4,49 @@
  *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
-#include <QJsonValue>
-#include <QJsonObject>
-#include <QJSValue>
+#include "pagerouter.h"
+#include "loggingcategory.h"
 #include <QJSEngine>
+#include <QJSValue>
+#include <QJsonObject>
+#include <QJsonValue>
 #include <QQmlProperty>
 #include <QQuickWindow>
-#include "pagerouter.h"
+#include <QTimer>
+#include <qqmlpropertymap.h>
 
-ParsedRoute* parseRoute(QJSValue value)
+ParsedRoute *parseRoute(QJSValue value)
 {
     if (value.isUndefined()) {
         return new ParsedRoute{};
     } else if (value.isString()) {
-        return new ParsedRoute{
-            value.toString(),
-            QVariant()
-        };
+        return new ParsedRoute{value.toString(), QVariant()};
     } else {
         auto map = value.toVariant().value<QVariantMap>();
         map.remove(QStringLiteral("route"));
         map.remove(QStringLiteral("data"));
-        return new ParsedRoute{
-            value.property(QStringLiteral("route")).toString(),
-            value.property(QStringLiteral("data")).toVariant(),
-            map,
-            false
-        };
+        return new ParsedRoute{value.property(QStringLiteral("route")).toString(), //
+                               value.property(QStringLiteral("data")).toVariant(),
+                               map,
+                               false};
     }
 }
 
-QList<ParsedRoute*> parseRoutes(QJSValue values)
+QList<ParsedRoute *> parseRoutes(QJSValue values)
 {
-    QList<ParsedRoute*> ret;
+    QList<ParsedRoute *> ret;
     if (values.isArray()) {
         const auto valuesList = values.toVariant().toList();
         for (const auto &route : valuesList) {
             if (route.toString() != QString()) {
-                ret << new ParsedRoute{
-                    route.toString(),
-                    QVariant(),
-                    QVariantMap(),
-                    false,
-                    nullptr
-                };
+                ret << new ParsedRoute{route.toString(), QVariant(), QVariantMap(), false, nullptr};
             } else if (route.canConvert<QVariantMap>()) {
                 auto map = route.value<QVariantMap>();
                 auto copy = map;
                 copy.remove(QStringLiteral("route"));
                 copy.remove(QStringLiteral("data"));
 
-                ret << new ParsedRoute{
-                    map.value(QStringLiteral("route")).toString(),
-                    map.value(QStringLiteral("data")),
-                    copy,
-                    false,
-                    nullptr
-                };
+                ret << new ParsedRoute{map.value(QStringLiteral("route")).toString(), map.value(QStringLiteral("data")), copy, false, nullptr};
             }
         }
     } else {
@@ -69,7 +55,11 @@ QList<ParsedRoute*> parseRoutes(QJSValue values)
     return ret;
 }
 
-PageRouter::PageRouter(QQuickItem *parent) : QObject(parent), m_cache(), m_preload()
+PageRouter::PageRouter(QQuickItem *parent)
+    : QObject(parent)
+    , m_paramMap(new QQmlPropertyMap)
+    , m_cache()
+    , m_preload()
 {
     connect(this, &PageRouter::pageStackChanged, [=]() {
         connect(m_pageStack, &ColumnView::currentIndexChanged, this, &PageRouter::currentIndexChanged);
@@ -81,41 +71,52 @@ QQmlListProperty<PageRoute> PageRouter::routes()
     return QQmlListProperty<PageRoute>(this, nullptr, appendRoute, routeCount, route, clearRoutes);
 }
 
-void PageRouter::appendRoute(QQmlListProperty<PageRoute>* prop, PageRoute* route)
+void PageRouter::appendRoute(QQmlListProperty<PageRoute> *prop, PageRoute *route)
 {
-    auto router = qobject_cast<PageRouter*>(prop->object);
+    auto router = qobject_cast<PageRouter *>(prop->object);
     router->m_routes.append(route);
 }
 
-int PageRouter::routeCount(QQmlListProperty<PageRoute>* prop)
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+int PageRouter::routeCount(QQmlListProperty<PageRoute> *prop)
+#else
+qsizetype PageRouter::routeCount(QQmlListProperty<PageRoute> *prop)
+#endif
 {
-    auto router = qobject_cast<PageRouter*>(prop->object);
+    auto router = qobject_cast<PageRouter *>(prop->object);
     return router->m_routes.length();
 }
 
-PageRoute* PageRouter::route(QQmlListProperty<PageRoute>* prop, int index)
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+PageRoute *PageRouter::route(QQmlListProperty<PageRoute> *prop, int index)
+#else
+PageRoute *PageRouter::route(QQmlListProperty<PageRoute> *prop, qsizetype index)
+#endif
 {
-    auto router = qobject_cast<PageRouter*>(prop->object);
+    auto router = qobject_cast<PageRouter *>(prop->object);
     return router->m_routes[index];
 }
 
-void PageRouter::clearRoutes(QQmlListProperty<PageRoute>* prop)
+void PageRouter::clearRoutes(QQmlListProperty<PageRoute> *prop)
 {
-    auto router = qobject_cast<PageRouter*>(prop->object);
+    auto router = qobject_cast<PageRouter *>(prop->object);
     router->m_routes.clear();
 }
 
-PageRouter::~PageRouter() {}
+PageRouter::~PageRouter()
+{
+}
 
 void PageRouter::classBegin()
 {
-
 }
 
 void PageRouter::componentComplete()
 {
     if (m_pageStack == nullptr) {
-        qCritical() << "PageRouter should be created with a ColumnView. Not doing so is undefined behaviour, and is likely to result in a crash upon further interaction.";
+        qCCritical(KirigamiLog)
+            << "PageRouter should be created with a ColumnView. Not doing so is undefined behaviour, and is likely to result in a crash upon further "
+               "interaction.";
     } else {
         Q_EMIT pageStackChanged();
         m_currentRoutes.clear();
@@ -126,15 +127,19 @@ void PageRouter::componentComplete()
 bool PageRouter::routesContainsKey(const QString &key) const
 {
     for (auto route : m_routes) {
-        if (route->name() == key) return true;
+        if (route->name() == key) {
+            return true;
+        }
     }
     return false;
 }
 
-QQmlComponent* PageRouter::routesValueForKey(const QString &key) const
+QQmlComponent *PageRouter::routesValueForKey(const QString &key) const
 {
     for (auto route : m_routes) {
-        if (route->name() == key) return route->component();
+        if (route->name() == key) {
+            return route->component();
+        }
     }
     return nullptr;
 }
@@ -142,7 +147,9 @@ QQmlComponent* PageRouter::routesValueForKey(const QString &key) const
 bool PageRouter::routesCacheForKey(const QString &key) const
 {
     for (auto route : m_routes) {
-        if (route->name() == key) return route->cache();
+        if (route->name() == key) {
+            return route->cache();
+        }
     }
     return false;
 }
@@ -150,25 +157,54 @@ bool PageRouter::routesCacheForKey(const QString &key) const
 int PageRouter::routesCostForKey(const QString &key) const
 {
     for (auto route : m_routes) {
-        if (route->name() == key) return route->cost();
+        if (route->name() == key) {
+            return route->cost();
+        }
     }
     return -1;
 }
 
-void PageRouter::push(ParsedRoute* route)
+// It would be nice if this could surgically update the
+// param map instead of doing this brute force approach,
+// but this seems to work well enough, and prematurely
+// optimising stuff is pretty bad if it isn't found as
+// a performance bottleneck.
+void PageRouter::reevaluateParamMapProperties()
+{
+    QStringList currentKeys;
+
+    for (auto item : m_currentRoutes) {
+        for (auto key : item->properties.keys()) {
+            currentKeys << key;
+
+            auto &value = item->properties[key];
+            m_paramMap->insert(key, value);
+        }
+    }
+
+    for (auto key : m_paramMap->keys()) {
+        if (!currentKeys.contains(key)) {
+            m_paramMap->clear(key);
+        }
+    }
+}
+
+void PageRouter::push(ParsedRoute *route)
 {
     Q_ASSERT(route);
     if (!routesContainsKey(route->name)) {
-        qCritical() << "Route" << route->name << "not defined";
+        qCCritical(KirigamiLog) << "Route" << route->name << "not defined";
         return;
     }
     if (routesCacheForKey(route->name)) {
-        auto push = [route, this](ParsedRoute* item) {
+        auto push = [route, this](ParsedRoute *item) {
             m_currentRoutes << item;
 
-            for ( auto it = route->properties.begin(); it != route->properties.end(); it++ ) {
+            for (auto it = route->properties.begin(); it != route->properties.end(); it++) {
                 item->item->setProperty(qUtf8Printable(it.key()), it.value());
+                item->properties[it.key()] = it.value();
             }
+            reevaluateParamMapProperties();
 
             m_pageStack->addItem(item->item);
         };
@@ -190,22 +226,27 @@ void PageRouter::push(ParsedRoute* route)
         // for a PageRouterAttached to find its parent
         // on construction time.
         auto item = component->beginCreate(context);
-        item->setParent(this);
-        auto qqItem = qobject_cast<QQuickItem*>(item);
-        if (!qqItem) {
-            qCritical() << "Route" << route->name << "is not an item! This is undefined behaviour and will likely crash your application.";
+        if (item == nullptr) {
+            return;
         }
-        for ( auto it = route->properties.begin(); it != route->properties.end(); it++ ) {
+        item->setParent(this);
+        auto qqItem = qobject_cast<QQuickItem *>(item);
+        if (!qqItem) {
+            qCCritical(KirigamiLog) << "Route" << route->name << "is not an item! This is undefined behaviour and will likely crash your application.";
+        }
+        for (auto it = route->properties.begin(); it != route->properties.end(); it++) {
             qqItem->setProperty(qUtf8Printable(it.key()), it.value());
         }
         route->setItem(qqItem);
         route->cache = routesCacheForKey(route->name);
         m_currentRoutes << route;
-        auto attached = qobject_cast<PageRouterAttached*>(qmlAttachedPropertiesObject<PageRouter>(item, true));
+        reevaluateParamMapProperties();
+
+        auto attached = qobject_cast<PageRouterAttached *>(qmlAttachedPropertiesObject<PageRouter>(item, true));
         attached->m_router = this;
         component->completeCreate();
         m_pageStack->addItem(qqItem);
-        m_pageStack->setCurrentIndex(m_currentRoutes.length()-1);
+        m_pageStack->setCurrentIndex(m_currentRoutes.length() - 1);
     };
 
     if (component->status() == QQmlComponent::Ready) {
@@ -214,12 +255,12 @@ void PageRouter::push(ParsedRoute* route)
         connect(component, &QQmlComponent::statusChanged, [=](QQmlComponent::Status status) {
             // Loading can only go to Ready or Error.
             if (status != QQmlComponent::Ready) {
-                qCritical() << "Failed to push route:" << component->errors();
+                qCCritical(KirigamiLog) << "Failed to push route:" << component->errors();
             }
             createAndPush();
         });
     } else {
-        qCritical() << "Failed to push route:" << component->errors();
+        qCCritical(KirigamiLog) << "Failed to push route:" << component->errors();
     }
 }
 
@@ -236,13 +277,13 @@ void PageRouter::setInitialRoute(QJSValue value)
 void PageRouter::navigateToRoute(QJSValue route)
 {
     auto incomingRoutes = parseRoutes(route);
-    QList<ParsedRoute*> resolvedRoutes;
+    QList<ParsedRoute *> resolvedRoutes;
 
     if (incomingRoutes.length() <= m_currentRoutes.length()) {
         resolvedRoutes = m_currentRoutes.mid(0, incomingRoutes.length());
     } else {
         resolvedRoutes = m_currentRoutes;
-        resolvedRoutes.reserve(incomingRoutes.length()-m_currentRoutes.length());
+        resolvedRoutes.reserve(incomingRoutes.length() - m_currentRoutes.length());
     }
 
     for (int i = 0; i < incomingRoutes.length(); i++) {
@@ -250,16 +291,21 @@ void PageRouter::navigateToRoute(QJSValue route)
         Q_ASSERT(incoming);
         if (i >= resolvedRoutes.length()) {
             resolvedRoutes.append(incoming);
-        } else { 
+        } else {
             auto current = resolvedRoutes.value(i);
             Q_ASSERT(current);
+            auto props = incoming->properties;
             if (current->name != incoming->name || current->data != incoming->data) {
                 resolvedRoutes.replace(i, incoming);
             }
+            resolvedRoutes[i]->properties.clear();
+            for (auto it = props.constBegin(); it != props.constEnd(); it++) {
+                resolvedRoutes[i]->properties.insert(it.key(), it.value());
+            }
         }
     }
 
-    for (const auto &route : qAsConst(m_currentRoutes)) {
+    for (const auto &route : std::as_const(m_currentRoutes)) {
         if (!resolvedRoutes.contains(route)) {
             placeInCache(route);
         }
@@ -267,9 +313,10 @@ void PageRouter::navigateToRoute(QJSValue route)
 
     m_pageStack->clear();
     m_currentRoutes.clear();
-    for (auto toPush : qAsConst(resolvedRoutes)) {
+    for (auto toPush : std::as_const(resolvedRoutes)) {
         push(toPush);
     }
+    reevaluateParamMapProperties();
     Q_EMIT navigationChanged();
 }
 
@@ -281,14 +328,14 @@ void PageRouter::bringToView(QJSValue route)
     } else {
         auto parsed = parseRoute(route);
         auto index = 0;
-        for (auto currentRoute : qAsConst(m_currentRoutes)) {
+        for (auto currentRoute : std::as_const(m_currentRoutes)) {
             if (currentRoute->name == parsed->name && currentRoute->data == parsed->data) {
                 m_pageStack->setCurrentIndex(index);
                 return;
             }
             index++;
         }
-        qWarning() << "Route" << parsed->name << "with data" << parsed->data << "is not on the current stack of routes.";
+        qCWarning(KirigamiLog) << "Route" << parsed->name << "with data" << parsed->data << "is not on the current stack of routes.";
     }
 }
 
@@ -322,21 +369,22 @@ void PageRouter::popRoute()
     m_pageStack->pop(m_currentRoutes.last()->item);
     placeInCache(m_currentRoutes.last());
     m_currentRoutes.removeLast();
+    reevaluateParamMapProperties();
     Q_EMIT navigationChanged();
 }
 
 QVariant PageRouter::dataFor(QObject *object)
 {
     auto pointer = object;
-    auto qqiPointer = qobject_cast<QQuickItem*>(object);
-    QHash<QQuickItem*,ParsedRoute*> routes;
-    for (auto route : qAsConst(m_cache.items)) {
+    auto qqiPointer = qobject_cast<QQuickItem *>(object);
+    QHash<QQuickItem *, ParsedRoute *> routes;
+    for (auto route : std::as_const(m_cache.items)) {
         routes[route->item] = route;
     }
-    for (auto route : qAsConst(m_preload.items)) {
+    for (auto route : std::as_const(m_preload.items)) {
         routes[route->item] = route;
     }
-    for (auto route : qAsConst(m_currentRoutes)) {
+    for (auto route : std::as_const(m_currentRoutes)) {
         routes[route->item] = route;
     }
     while (qqiPointer != nullptr) {
@@ -365,7 +413,7 @@ bool PageRouter::isActive(QObject *object)
     auto pointer = object;
     while (pointer != nullptr) {
         auto index = 0;
-        for (auto route : qAsConst(m_currentRoutes)) {
+        for (auto route : std::as_const(m_currentRoutes)) {
             if (route->item == pointer) {
                 return m_pageStack->currentIndex() == index;
             }
@@ -373,26 +421,26 @@ bool PageRouter::isActive(QObject *object)
         }
         pointer = pointer->parent();
     }
-    qWarning() << "Object" << object << "not in current routes";
+    qCWarning(KirigamiLog) << "Object" << object << "not in current routes";
     return false;
 }
 
-PageRouterAttached* PageRouter::qmlAttachedProperties(QObject *object)
+PageRouterAttached *PageRouter::qmlAttachedProperties(QObject *object)
 {
     auto attached = new PageRouterAttached(object);
     return attached;
 }
 
-QSet<QObject*> flatParentTree(QObject* object)
+QSet<QObject *> flatParentTree(QObject *object)
 {
     // See below comment in Climber::climbObjectParents for why this is here.
-    static const QMetaObject* metaObject = QMetaType::metaObjectForType(QMetaType::type("QQuickItem*"));
-    QSet<QObject*> ret;
+    static const QMetaObject *metaObject = QMetaType::metaObjectForType(QMetaType::type("QQuickItem*"));
+    QSet<QObject *> ret;
     // Use an inline struct type so that climbItemParents and climbObjectParents
-    // can call eachother
-    struct Climber
-    {
-        void climbItemParents(QSet<QObject*> &out, QQuickItem *item) {
+    // can call each other
+    struct Climber {
+        void climbItemParents(QSet<QObject *> &out, QQuickItem *item)
+        {
             auto parent = item->parentItem();
             while (parent != nullptr) {
                 out << parent;
@@ -400,39 +448,40 @@ QSet<QObject*> flatParentTree(QObject* object)
                 parent = parent->parentItem();
             }
         }
-        void climbObjectParents(QSet<QObject*> &out, QObject *object) {
+        void climbObjectParents(QSet<QObject *> &out, QObject *object)
+        {
             auto parent = object->parent();
             while (parent != nullptr) {
                 out << parent;
                 // We manually call metaObject()->inherits() and
                 // use a reinterpret cast because qobject_cast seems
-                // to have stability issues here due to mutable 
+                // to have stability issues here due to mutable
                 // pointer mechanics.
                 if (parent->metaObject()->inherits(metaObject)) {
-                    climbItemParents(out, reinterpret_cast<QQuickItem*>(parent));
+                    climbItemParents(out, reinterpret_cast<QQuickItem *>(parent));
                 }
                 parent = parent->parent();
             }
         }
     };
     Climber climber;
-    if (qobject_cast<QQuickItem*>(object)) {
-        climber.climbItemParents(ret, qobject_cast<QQuickItem*>(object));
+    if (qobject_cast<QQuickItem *>(object)) {
+        climber.climbItemParents(ret, qobject_cast<QQuickItem *>(object));
     }
     climber.climbObjectParents(ret, object);
     return ret;
 }
 
-void PageRouter::preload(ParsedRoute* route)
+void PageRouter::preload(ParsedRoute *route)
 {
-    for (auto preloaded : qAsConst(m_preload.items)) {
+    for (auto preloaded : std::as_const(m_preload.items)) {
         if (preloaded->equals(route)) {
             delete route;
             return;
         }
     }
     if (!routesContainsKey(route->name)) {
-        qCritical() << "Route" << route->name << "not defined";
+        qCCritical(KirigamiLog) << "Route" << route->name << "not defined";
         delete route;
         return;
     }
@@ -441,20 +490,20 @@ void PageRouter::preload(ParsedRoute* route)
     auto createAndCache = [component, context, route, this]() {
         auto item = component->beginCreate(context);
         item->setParent(this);
-        auto qqItem = qobject_cast<QQuickItem*>(item);
+        auto qqItem = qobject_cast<QQuickItem *>(item);
         if (!qqItem) {
-            qCritical() << "Route" << route->name << "is not an item! This is undefined behaviour and will likely crash your application.";
+            qCCritical(KirigamiLog) << "Route" << route->name << "is not an item! This is undefined behaviour and will likely crash your application.";
         }
-        for ( auto it = route->properties.begin(); it != route->properties.end(); it++ ) {
+        for (auto it = route->properties.begin(); it != route->properties.end(); it++) {
             qqItem->setProperty(qUtf8Printable(it.key()), it.value());
         }
         route->setItem(qqItem);
         route->cache = routesCacheForKey(route->name);
-        auto attached = qobject_cast<PageRouterAttached*>(qmlAttachedPropertiesObject<PageRouter>(item, true));
+        auto attached = qobject_cast<PageRouterAttached *>(qmlAttachedPropertiesObject<PageRouter>(item, true));
         attached->m_router = this;
         component->completeCreate();
         if (!route->cache) {
-            qCritical() << "Route" << route->name << "is being preloaded despite it not having caching enabled.";
+            qCCritical(KirigamiLog) << "Route" << route->name << "is being preloaded despite it not having caching enabled.";
             delete route;
             return;
         }
@@ -469,19 +518,19 @@ void PageRouter::preload(ParsedRoute* route)
         connect(component, &QQmlComponent::statusChanged, [=](QQmlComponent::Status status) {
             // Loading can only go to Ready or Error.
             if (status != QQmlComponent::Ready) {
-                qCritical() << "Failed to push route:" << component->errors();
+                qCCritical(KirigamiLog) << "Failed to push route:" << component->errors();
             }
             createAndCache();
         });
     } else {
-        qCritical() << "Failed to push route:" << component->errors();
+        qCCritical(KirigamiLog) << "Failed to push route:" << component->errors();
     }
 }
 
-void PageRouter::unpreload(ParsedRoute* route)
+void PageRouter::unpreload(ParsedRoute *route)
 {
-    ParsedRoute* toDelete = nullptr;
-    for (auto preloaded : qAsConst(m_preload.items)) {
+    ParsedRoute *toDelete = nullptr;
+    for (auto preloaded : std::as_const(m_preload.items)) {
         if (preloaded->equals(route)) {
             toDelete = preloaded;
         }
@@ -496,7 +545,7 @@ void PageRouter::unpreload(ParsedRoute* route)
 void PreloadRouteGroup::handleChange()
 {
     if (!(m_parent->m_router)) {
-        qCritical() << "PreloadRouteGroup does not have a parent PageRouter";
+        qCCritical(KirigamiLog) << "PreloadRouteGroup does not have a parent PageRouter";
         return;
     }
     auto r = m_parent->m_router;
@@ -519,7 +568,7 @@ void PageRouterAttached::findParent()
 {
     QQuickItem *parent = qobject_cast<QQuickItem *>(this->parent());
     while (parent != nullptr) {
-        auto attached = qobject_cast<PageRouterAttached*>(qmlAttachedPropertiesObject<PageRouter>(parent, false));
+        auto attached = qobject_cast<PageRouterAttached *>(qmlAttachedPropertiesObject<PageRouter>(parent, false));
         if (attached != nullptr && attached->m_router != nullptr) {
             m_router = attached->m_router;
             Q_EMIT routerChanged();
@@ -537,7 +586,7 @@ void PageRouterAttached::navigateToRoute(QJSValue route)
     if (m_router) {
         m_router->navigateToRoute(route);
     } else {
-        qCritical() << "PageRouterAttached does not have a parent PageRouter";
+        qCCritical(KirigamiLog) << "PageRouterAttached does not have a parent PageRouter";
         return;
     }
 }
@@ -547,7 +596,7 @@ bool PageRouterAttached::routeActive(QJSValue route)
     if (m_router) {
         return m_router->routeActive(route);
     } else {
-        qCritical() << "PageRouterAttached does not have a parent PageRouter";
+        qCCritical(KirigamiLog) << "PageRouterAttached does not have a parent PageRouter";
         return false;
     }
 }
@@ -557,7 +606,7 @@ void PageRouterAttached::pushRoute(QJSValue route)
     if (m_router) {
         m_router->pushRoute(route);
     } else {
-        qCritical() << "PageRouterAttached does not have a parent PageRouter";
+        qCCritical(KirigamiLog) << "PageRouterAttached does not have a parent PageRouter";
         return;
     }
 }
@@ -567,7 +616,7 @@ void PageRouterAttached::popRoute()
     if (m_router) {
         m_router->popRoute();
     } else {
-        qCritical() << "PageRouterAttached does not have a parent PageRouter";
+        qCCritical(KirigamiLog) << "PageRouterAttached does not have a parent PageRouter";
         return;
     }
 }
@@ -577,7 +626,7 @@ void PageRouterAttached::bringToView(QJSValue route)
     if (m_router) {
         m_router->bringToView(route);
     } else {
-        qCritical() << "PageRouterAttached does not have a parent PageRouter";
+        qCCritical(KirigamiLog) << "PageRouterAttached does not have a parent PageRouter";
         return;
     }
 }
@@ -587,7 +636,7 @@ QVariant PageRouterAttached::data() const
     if (m_router) {
         return m_router->dataFor(parent());
     } else {
-        qCritical() << "PageRouterAttached does not have a parent PageRouter";
+        qCCritical(KirigamiLog) << "PageRouterAttached does not have a parent PageRouter";
         return QVariant();
     }
 }
@@ -597,7 +646,7 @@ bool PageRouterAttached::isCurrent() const
     if (m_router) {
         return m_router->isActive(parent());
     } else {
-        qCritical() << "PageRouterAttached does not have a parent PageRouter";
+        qCCritical(KirigamiLog) << "PageRouterAttached does not have a parent PageRouter";
         return false;
     }
 }
@@ -607,7 +656,7 @@ bool PageRouterAttached::watchedRouteActive()
     if (m_router) {
         return m_router->routeActive(m_watchedRoute);
     } else {
-        qCritical() << "PageRouterAttached does not have a parent PageRouter";
+        qCCritical(KirigamiLog) << "PageRouterAttached does not have a parent PageRouter";
         return false;
     }
 }
@@ -628,7 +677,7 @@ void PageRouterAttached::pushFromHere(QJSValue route)
     if (m_router) {
         m_router->pushFromObject(parent(), route);
     } else {
-        qCritical() << "PageRouterAttached does not have a parent PageRouter";
+        qCCritical(KirigamiLog) << "PageRouterAttached does not have a parent PageRouter";
     }
 }
 
@@ -637,7 +686,7 @@ void PageRouterAttached::replaceFromHere(QJSValue route)
     if (m_router) {
         m_router->pushFromObject(parent(), route, true);
     } else {
-        qCritical() << "PageRouterAttached does not have a parent PageRouter";
+        qCCritical(KirigamiLog) << "PageRouterAttached does not have a parent PageRouter";
     }
 }
 
@@ -646,7 +695,7 @@ void PageRouterAttached::popFromHere()
     if (m_router) {
         m_router->pushFromObject(parent(), QJSValue());
     } else {
-        qCritical() << "PageRouterAttached does not have a parent PageRouter";
+        qCCritical(KirigamiLog) << "PageRouterAttached does not have a parent PageRouter";
     }
 }
 
@@ -667,11 +716,12 @@ void PageRouter::pushFromObject(QObject *object, QJSValue inputRoute, bool repla
     const auto parsed = parseRoutes(inputRoute);
     const auto objects = flatParentTree(object);
 
-    for (const auto& obj : objects) {
+    for (const auto &obj : objects) {
         bool popping = false;
-        for (auto route : qAsConst(m_currentRoutes)) {
+        for (auto route : std::as_const(m_currentRoutes)) {
             if (popping) {
                 m_currentRoutes.removeAll(route);
+                reevaluateParamMapProperties();
                 placeInCache(route);
                 continue;
             }
@@ -679,6 +729,7 @@ void PageRouter::pushFromObject(QObject *object, QJSValue inputRoute, bool repla
                 m_pageStack->pop(route->item);
                 if (replace) {
                     m_currentRoutes.removeAll(route);
+                    reevaluateParamMapProperties();
                     m_pageStack->removeItem(route->item);
                 }
                 popping = true;
@@ -694,7 +745,7 @@ void PageRouter::pushFromObject(QObject *object, QJSValue inputRoute, bool repla
             return;
         }
     }
-    qWarning() << "Object" << object << "not in current routes";
+    qCWarning(KirigamiLog) << "Object" << object << "not in current routes";
 }
 
 QJSValue PageRouter::currentRoutes() const
@@ -705,15 +756,21 @@ QJSValue PageRouter::currentRoutes() const
         auto object = engine->newObject();
         object.setProperty(QStringLiteral("route"), m_currentRoutes[i]->name);
         object.setProperty(QStringLiteral("data"), engine->toScriptValue(m_currentRoutes[i]->data));
+        auto keys = m_currentRoutes[i]->properties.keys();
+        for (auto key : keys) {
+            object.setProperty(key, engine->toScriptValue(m_currentRoutes[i]->properties[key]));
+        }
         ret.setProperty(i, object);
     }
     return ret;
 }
 
-PageRouterAttached::PageRouterAttached(QObject *parent) : QObject(parent), m_preload(new PreloadRouteGroup(this))
+PageRouterAttached::PageRouterAttached(QObject *parent)
+    : QObject(parent)
+    , m_preload(new PreloadRouteGroup(this))
 {
     findParent();
-    auto item = qobject_cast<QQuickItem*>(parent);
+    auto item = qobject_cast<QQuickItem *>(parent);
     if (item != nullptr) {
         connect(item, &QQuickItem::windowChanged, this, [this]() {
             findParent();
